import re
from os import environ
from datetime import datetime

from mkdocs.config import config_options
from mkdocs.plugins import BasePlugin
from .util import Util


class GitRevisionDatePlugin(BasePlugin):
    config_scheme = (
        ('enabled_if_env', config_options.Type(str)),
        ('modify_md', config_options.Type(bool, default=True)),
        ('as_datetime', config_options.Type(bool, default=False)),
    )

    def __init__(self):
        self.enabled = True
        self.util = Util()

    def on_config(self, config):
        env_name = self.config['enabled_if_env']
        if env_name:
            self.enabled = environ.get(env_name) == '1'
            if not self.enabled:
                print('PDF export is disabled (set environment variable %s to 1 to enable)' % env_name)
                return

    def on_page_markdown(self, markdown, page, config, files):
        if not self.enabled:
            return markdown

        revision_date = self.util.get_revision_date_for_file(page.file.abs_src_path)


        # TODO: if the file has no git logs, it could be empty. default to date the file was created instead?
        if not revision_date:
            revision_date = datetime.now().date().strftime('%Y-%m-%d')
            print('WARNING -  %s has no git logs, revision date defaulting to today\'s date' % page.file.src_path)

        if self.config['as_datetime']:
            revision_date = datetime.strptime(revision_date,'%Y-%m-%d')

        page.meta['revision_date'] = revision_date

        if not self.config['modify_md']:
            return markdown

        if 'macros' in config['plugins']:
            keys = list(config['plugins'].keys())
            vals = list(config['plugins'].values())
            if keys.index('macros') > vals.index(self):
                new_markdown = '{{% set git_revision_date = \'{}\' %}}\n'.format(revision_date) + markdown
                return new_markdown
            else:
                print('WARNING - macros plugin must be placed AFTER the git-revision-date plugin. Skipping markdown modifications')
                return markdown
        else:
            markdown = re.sub(r"\{\{(\s)*git_revision_date(\s)*\}\}",
                          str(revision_date),
                          markdown,
                          flags=re.IGNORECASE)

            markdown = re.sub(r"\{\{\s*page\.meta\.git_revision_date\s*\}\}",
                          str(revision_date),
                          markdown,
                          flags=re.IGNORECASE)
            
            return markdown



import git
import git.repo
import datetime

myRepo = git.repo.Repo("C:/Users/Cody/tuas/wiki")

# print(myRepo.commit('bdea42410c5655e1e9505b3d87099897716987a0').)

# iterator for each blame 'chunk' of the file
blameIter = myRepo.blame_incremental(myRepo.head, "docs/general/repo_guide.md")
currOutput = myRepo.blame(myRepo.head, "docs/embedded/electronics-diagram.md")

# print each timestamp of the git blame

blameTimes = []

for blame in blameIter:
    rawEpochTime = blame.commit.authored_date # type: ignore (not sure why this gives an error; it works fine)
    timeSince = datetime.datetime.fromtimestamp(rawEpochTime).strftime('%Y-%m-%d') # add %H:%M:%S for specific time
    blameTimes.append(timeSince)

# print(blameTimes)

def getMostRecentTime(times):

    mostRecentTime = '0000-00-00'

    mostRecentTimeYear = int(mostRecentTime[0:4])
    mostRecentTimeMonth = int(mostRecentTime[5:7])
    mostRecentTimeDay = int(mostRecentTime[8:10])

    for time in times:
        year = int(time[0:4])
        month = int(time[5:7])
        day = int(time[8:10])
        # print(year)

        if year > mostRecentTimeYear:
            # print('yes')
            mostRecentTime = time
        elif (year == mostRecentTimeYear and month > mostRecentTimeMonth):
            mostRecentTime = time
        elif (year == mostRecentTimeYear and month == mostRecentTimeMonth and day > mostRecentTimeDay):
            mostRecentTime = time

        mostRecentTimeYear = int(mostRecentTime[0:4])
        mostRecentTimeMonth = int(mostRecentTime[5:7])
        mostRecentTimeDay = int(mostRecentTime[8:10])

    # print(mostRecentTimeYear)

    # have to do this case because windows can't use the %-d strftime code. %e is the windows equivalent,
    #   but instead of actually removing the 0 padding, it prepends the number with a space, leaving some awkward gaps when the number has 2 digits.
    #   tldr; gg windows sucks
    if (mostRecentTimeDay < 10):
        return datetime.date(mostRecentTimeYear,mostRecentTimeMonth,mostRecentTimeDay).strftime("%B%e, %Y")

    return datetime.date(mostRecentTimeYear,mostRecentTimeMonth,mostRecentTimeDay).strftime("%B %d, %Y")

print('most recent time:', getMostRecentTime(blameTimes))


from setuptools import setup, find_packages

setup(
    name='mkdocs-git-revision-date-plugin-blame',
    version='0.1.6',
    description='MkDocs plugin for setting revision date from git per markdown file using blame.',
    keywords='mkdocs git meta yaml frontmatter',
    url='https://github.com/codyprupp/mkdocs-git-revision-date-plugin',
    author='Cody Rupp',
    author_email='crupp@ucsd.edu',
    license='MIT',
    license_files = ('LICENSE'),
    python_requires='>=3.4',
    install_requires=[
        'mkdocs>=0.17',
        'GitPython',
        'jinja2'
    ],
    packages=["mkdocs_git_revision_date_plugin"],
    entry_points={
        'mkdocs.plugins': [
            'git-revision-date-blame = mkdocs_git_revision_date_plugin.plugin:GitRevisionDatePlugin'
        ]
    }
)



from git.cmd import Git
import git.repo
import datetime
import os

class Util:

    def __init__(self):
        self.g = Git()

    # def get_revision_date_for_file(self, path: str):
    #     return self.g.log(path, n=1, date='short', format='%ad')
    

    # NOTE: this implies that a file is "up to date" if the most recent git commit displays a somewhat recent date.
    #   This is not entirely accurate, unless whoever made that commit checked the entire file for discrepancies, which is not always the case.
    #   To fix this, either display a verbose output with every single line of blame, or figure out a way to display the blame for a specific
    #       while hovering over the corresponding text.
    def get_revision_date_for_file(self, path: str):

        # get the raw blame for the file
        repo = git.repo.Repo(os.getcwd())
        blameIter = repo.blame_incremental(repo.head, path)

        blameTimes = []

        # extract each blame's date in the form YYYY-MM-DD
        for blame in blameIter:
            rawEpochTime = blame.commit.authored_date # type: ignore (not sure why this gives an error; it works fine)
            timeSince = datetime.datetime.fromtimestamp(rawEpochTime).strftime('%Y-%m-%d') # add %H:%M:%S for specific time
            blameTimes.append(timeSince)

        mostRecentTime = '0000-00-00'

        mostRecentTimeYear = int(mostRecentTime[0:4])
        mostRecentTimeMonth = int(mostRecentTime[5:7])
        mostRecentTimeDay = int(mostRecentTime[8:10])

        # find most recent blame (to be displayed as last updated)
        for time in blameTimes:
            year = int(time[0:4])
            month = int(time[5:7])
            day = int(time[8:10])

            if year > mostRecentTimeYear:
                mostRecentTime = time
            elif (year == mostRecentTimeYear and month > mostRecentTimeMonth):
                mostRecentTime = time
            elif (year == mostRecentTimeYear and month == mostRecentTimeMonth and day > mostRecentTimeDay):
                mostRecentTime = time

            mostRecentTimeYear = int(mostRecentTime[0:4])
            mostRecentTimeMonth = int(mostRecentTime[5:7])
            mostRecentTimeDay = int(mostRecentTime[8:10])


        # have to do this case because windows can't use the %-d strftime code. %e is the windows equivalent,
        #   but instead of actually removing the 0 padding, it prepends the number with a space, leaving some awkward gaps when the number has 2 digits.
        #   tldr; gg windows sucks
        if (mostRecentTimeDay < 10):
            return datetime.date(mostRecentTimeYear,mostRecentTimeMonth,mostRecentTimeDay).strftime("%B%e, %Y")

        return datetime.date(mostRecentTimeYear,mostRecentTimeMonth,mostRecentTimeDay).strftime("%B %d, %Y")